// Interior Interaction Functions Library v2.1 Serena 24/12/2016
// ----------------------------------------------
// Gives ability to carry and stack interior furniture, boxes and similiar objects, and put weapons and ammunition on top of them
// To complete work of all features library requires enchanced movement and weapon holster functions, for example from Enhanced-Movement addon.
// Usage:
// 		unit call Eint_AddInteractionAbilities - to add unit interaction abilitues
//		["ObjectClassName", SurfaceOffset, MassOverride] call Eint_RequestRegistry - to register new interactible object class
//			SurfaceOffset - object top surface offset, or array of offsets, or nil to disable put items ability
//			MassOverride - new mass value in kg, or nil
//		[obj1, obj2, obj2] call Eint_OverrideObjectsMass - to override mass of objects to user defined values
// 		unit setVariable ["Ent_PutItemsPointer", "Sign_Arrow_Direction_Green_F" createVehicle [0,0,0]] - to enable put items marker

Eint_RequestRegistry = {
	switch (true) do {
		case (_this isEqualType ""): {{if (_x select 0 isEqualTo _this) exitWith {_x}} forEach Eint_ObjectRegistry};
		case (_this isEqualType []): {Eint_ObjectRegistry pushBack _this};
		default {_reg}}
};

Eint_PreparePutInteraction = {
	params ["_sdr", "_ptr"];
	private _res = nil;
	if (isPlayer _sdr and isNil {_sdr getVariable "Eint_InteractionRestricted"}) then {
		private _obj = {_x} forEach lineIntersectsObjs [(eyePos _sdr),(ATLtoASL screenToWorld [0.5, 0.5]),objNull,objNull,false, 2 + 8];
		if (not isNil "_obj") then {
			private _rec = typeOf _obj call Eint_RequestRegistry;
			if (not isNil "_rec") then {
				_rec params ["", "_hgt"];
				if (not isNil "_hgt") then {
					(boundingBoxReal _obj select 1) vectorDiff (boundingBoxReal _obj select 0) vectorMultiply 0.5 params ["_wdf", "_ddf", "_hdf"];
					private _iuh = _sdr modelToWorld [0.0, [0.3, 0.0] select (currentWeapon _sdr == ""), 1.0] vectorAdd		
						(vectorNormalized [getCameraViewDirection _sdr select 0, getCameraViewDirection _sdr select 1, 0] vectorMultiply 0.9);
					private _iud = _obj getRelDir _iuh;
					private _ost = _obj distance2D _iuh;
					if (_ost < vectorMagnitude [sin _iud * _wdf, cos _iud * _ddf, 0]) then {
						if (_hgt isEqualType []) then {
							private _sdh = _sdr modelToWorld (_sdr selectionPosition "head") select 2;	
							_hgt = {if (_sdh > (_obj modelToWorld [0, 0, _x] select 2)) exitWith {_x}; _x} forEach _hgt};
							
						if (not isNil "_ptr") then {
							_ptr setDir (getDir _sdr);
							_ptr setPosATL [_iuh select 0, _iuh select 1, _obj modelToWorld [0, 0, _hgt] select 2]};
							
						_res = [_sdr, _obj, [sin _iud * _ost, cos _iud * _ost, _hgt + 0.661233]];
					}
				}
			}
		}
	};
	if (not isNil "_ptr") then {
		_ptr hideObject (isNil "_res")};
	_res
};

Eint_BeginPutInteraction = {
	params ["_sdr", "_obj", "_atp"];
	private _gwh = "GroundWeaponHolder" createVehicle [0,0,0];
	_gwh attachTo [_obj, _atp];
	_gwh setVectorDirAndUp [[sin (getDir _sdr - getDir _obj - 90), cos (getDir _sdr - getDir _obj - 90), 0], [0,0,1]];
	_sdr action ["Gear", _gwh];
	_gwh addEventHandler ["ContainerClosed", { params ["_gwh", "_sdr"];
		if (count weaponCargo _gwh < 1 and {count itemCargo _gwh < 1} and {count magazineCargo _gwh < 1}) then {
			detach _gwh; deleteVehicle _gwh}}];
};

Eint_PrepareMoveInteraction = {
	params ["_sdr"];
	private _res = nil;
	if (isPlayer _sdr and isNil {_sdr getVariable "Eint_InteractionRestricted"}) then {
		private _obj = {_x} forEach lineIntersectsObjs [(eyePos _sdr),(ATLtoASL screenToWorld [0.5, 0.5]),objNull,objNull,false, 2 + 8];
		if (not isNil "_obj") then {
			private _rec = typeOf _obj call Eint_RequestRegistry;
			if (not isNil "_rec") then {
				(boundingBoxReal _obj select 1) vectorDiff (boundingBoxReal _obj select 0) vectorMultiply 0.5 params ["_wdf", "_ddf", "_hdf"];
				private _edd = vectorMagnitude [sin (_obj getRelDir _sdr) * _wdf, cos (_obj getRelDir _sdr) * _ddf, 0];
				private _ofs = _sdr worldToModel (_obj modelToWorld [0,0,0.30]) vectorDiff (_sdr selectionPosition "spine3");
				if (vectorMagnitude _ofs - _edd < 1.0) then {
					_res = [_sdr, _obj, _edd, _ofs]}
			}
		}
	};
	_res
};

Eint_BeginMoveInteraction = {
	params ["_sdr", "_obj", "_edd", "_ofs"];
	private _ang = getDir _obj - (_sdr getDir _obj);
	_sdr setVariable ["Eint_InteractionRestricted", true];
	_sdr setDir (_sdr getDir _obj);
	_obj attachTo [_sdr, _ofs, "spine3"];
	_obj setVectorDirAndUp [[sin _ang, cos _ang, 0], [0,0,1]];
	
	private _aid = _sdr addAction ["Release Object", {
		params ["_unt", "_sdr", "_aid", "_obj"];
		if (_unt == _sdr) then {
			detach _obj}
	}, _obj];
	
	[_sdr, _obj, _aid] spawn {
		params ["_sdr", "_obj", "_aid"];
		private _dmg = damage _sdr;
		while {alive _sdr and damage _sdr <= _dmg and attachedTo _obj == _sdr and getStamina _sdr > 1.0} do {
			private _elv = getPosASL _obj vectorDiff getPosASL _sdr select 2;
			if (_elv < 0.30 or _elv > 1.2) then {
				_obj attachTo [_sdr, _sdr worldToModel (_obj modelToWorld [0,0,0.30 - _elv]) vectorDiff (_sdr selectionPosition "spine3"), "spine3"]};
			sleep 0.1;
			_sdr setStamina (getStamina _sdr - (getMass _obj / 20) ^ 1.5  * 0.3 * (if (currentWeapon _sdr == "") then {1} else {1.5}));
		};
		_sdr removeAction _aid;
		detach _obj;
		_obj setVelocity ((velocity _sdr vectorMultiply 0.5) vectorAdd (vectorDir _sdr vectorMultiply 0.25) vectorAdd [0, 0, 0.2] );
		_sdr setVariable ["Eint_InteractionRestricted", nil];
	}
};

Eint_AddInteractionAbilities = {
	_this addAction ["Put Items", {((_this select 0) getVariable "Eint_PutItemsData") call Eint_BeginPutInteraction},
		nil, 1.5, true, true, "", "private _pid = [_this, _this getVariable 'Ent_PutItemsPointer']
			call Eint_PreparePutInteraction; _this setVariable ['Eint_PutItemsData', _pid]; not isNil '_pid'"];
			
	_this addAction ["Move Object", {((_this select 0) getVariable "Eint_MoveObjectData") call Eint_BeginMoveInteraction},
		nil, 1.5, true, true, "", "private _mod = _this call Eint_PrepareMoveInteraction;
			_this setVariable ['Eint_MoveObjectData', _mod]; not isNil '_mod'"]
};

Eint_OverrideObjectsMass = {		
	{	private _rec = typeOf _x call Eint_RequestRegistry;
		if (not isNil "_rec") then {
			_rec params ["", "", "_ovm"];
			if (not isNil "_ovm") then {
				_x setMass _ovm}}
	} forEach _this
};

Eint_ObjectRegistry = [
	["Land_CampingTable_F", 0.397, 25], ["Land_CampingTable_small_F", 0.397, 15], ["Land_CampingChair_V1_F", -0.017, 6], ["Land_CampingChair_V2_F", 0.006, 5],	
	["Land_TablePlastic_01_F", 0.430, 30], ["Land_ChairPlastic_F", -0.018, 5], ["Fridge_01_closed_F", 0.506, 30],
	["Land_RattanTable_01_F", 0.383, 12], ["Land_RattanChair_01_F", -0.039, 5],
	
	["Land_WoodenTable_large_F", 0.413, 35], ["Land_WoodenTable_small_F", 0.414, 25], ["Land_Bench_F", 0.250, 20], ["Land_WoodenCrate_01_F", 0.332, 20],

	["OfficeTable_01_old_F", 0.396, 30], ["Land_ChairWood_F", 0.511, 6],
	["OfficeTable_01_new_F", 0.396, 30], ["Land_TableDesk_F", 0.406, 30], ["Land_OfficeChair_01_F", -0.107, 12], ["Land_OfficeCabinet_01_F", [0.770, 0.386, 0.0675], 35],

	["Land_ShelvesMetal_F", [1.249, 0.897, 0.545, 0.185], 60], ["Land_ShelvesWooden_F", [0.457, 0.114, -0.198], 30],
	["Land_ShelvesWooden_blue_F", [0.457, 0.114, -0.198], 30], ["Land_ShelvesWooden_khaki_F", [0.457, 0.114, -0.198], 30],

	["Land_Metal_wooden_rack_F", [0.995, 0.569, 0.0700, -0.430, -0.930], 70], ["Land_Rack_F", [0.654, 0.279, -0.041, -0.398, -0.724], 40],
	["Land_Metal_rack_F", [0.539, 0.089, -0.361, -0.810], 35], ["Land_Metal_rack_Tall_F", [1.611, 1.230, 0.850, 0.470, 0.089], 30],

	["Land_PlasticCase_01_large_F", 0.361, 15], ["Land_PlasticCase_01_medium_F", 0.175, 10], ["Land_PlasticCase_01_small_F", 0.143, 5],
	["Land_MetalCase_01_large_F", 0.350, 15], ["Land_MetalCase_01_medium_F", 0.167, 10], ["Land_MetalCase_01_small_F", 0.134, 5],
	["Box_NATO_Ammo_F", 0.309, 25], ["Box_NATO_Wps_F", 0.179, 35], ["Box_NATO_WpsSpecial_F", 0.184, 35], ["CargoNet_01_box_F", 0.561, 250],
	
	["Land_MapBoard_F", nil, 15], ["Land_Camping_Light_F", nil, 1], ["Land_Camping_Light_off_F", nil, 1]];

true

